/* Parameters for execution on a NeXT, for GDB, the GNU debugger
   Copyright 1986, 1987, 1989, 1991, 1992, 1993 Free Software Foundation, Inc.

This file is part of GDB.

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */

#if !defined (TM_H)
#define TM_H 1

#ifdef NeXT
#include <mach/mach.h>
#endif	/* NeXT */

/*
 * Changes for 80386 by Pace Willisson (pace@prep.ai.mit.edu)
 * July 1988
 *
 * This file adapted from tm-i386v.h, with floating point from tm-sun386.h
 */

#define TARGET_BYTE_ORDER LITTLE_ENDIAN

#define IEEE_FLOAT 1

/* Define this if the C compiler puts an underscore at the front
   of external names before giving them to the linker.  */

/* #define NAMES_HAVE_UNDERSCORE */	/* not needed if we don't use BFD */

/* Offset from address of function to start of its code.
   Zero on most machines.  */

#define FUNCTION_START_OFFSET 0

/* Advance PC across any function entry prologue instructions
   to reach some "real" code.  */

#define SKIP_PROLOGUE(frompc)   {(frompc) = i386_skip_prologue((frompc));}

extern int
i386_skip_prologue PARAMS ((int));

#define MARK_PROLOGUE(f, s, e) mark_prologue(f, s, e)	/* NeXT specific */

#ifdef NeXT	/* SKIP_TRAMPOLINE stuff */
/* If PC is in some function-call trampoline code, return the PC
   where the function itself actually starts.  If not, return NULL.  */

extern CORE_ADDR skip_trampoline_code (CORE_ADDR pc, char *name);

#define	SKIP_TRAMPOLINE_CODE(pc) skip_trampoline_code (pc, NULL)

/* Return non-zero if we are in some sort of a trampoline. */

#define IN_SOLIB_TRAMPOLINE(pc, name) (isDylibJump(pc) || isShlibJump(pc))

#endif	/* NeXT SKIP_TRAMPOLINE */

/* Immediately after a function call, return the saved pc.
   Can't always go through the frames for this because on some machines
   the new frame is not set up until the new function executes
   some instructions.  */

#define SAVED_PC_AFTER_CALL(frame) \
  (read_memory_integer (read_register (SP_REGNUM), 4))

/* Stack grows downward.  */

#define INNER_THAN <

/* Sequence of bytes for breakpoint instruction.  */

#define BREAKPOINT {0xcc}
#define KERNEL_BREAKPOINT {0xcc}

/* Amount PC must be decremented by after a breakpoint.
   This is often the number of bytes in BREAKPOINT
   but not always.  */

#ifndef DECR_PC_AFTER_BREAK
#define DECR_PC_AFTER_BREAK 1
#endif

/* Nonzero if instruction at PC is a return instruction.  */

#define ABOUT_TO_RETURN(pc) (read_memory_integer (pc, 1) == 0xc3)

/* code to execute to print interesting information about the
   floating point processor (if any)
   No need to define if there is nothing to do.
   On the 386, unfortunately this code is host-dependent (and lives
   in the i386-xdep.c file), so we can't
   do this unless we *know* we aren't cross-debugging.  FIXME.
 */
#define FLOAT_INFO { i386_float_info (); }

/* Say how long (ordinary) registers are.  This is a piece of bogosity
   used in push_word and a few other places; REGISTER_RAW_SIZE is the
   real way to know how big a register is.  */

#define REGISTER_SIZE 4

/* Number of machine registers */

#define NUM_REGS 31

/* Initializer for an array of names of registers.
   There should be NUM_REGS strings in this initializer.  */

/* the order of the first 8 registers must match the compiler's 
 * numbering scheme (which is the same as the 386 scheme)
 * also, this table must match regmap in i386-pinsn.c.
 */
#define REGISTER_NAMES { "eax", "ecx", "edx", "ebx",      \
			 "esp", "ebp", "esi", "edi",      \
			 "eip", "eflags", "cs", "ss",     \
			 "ds", "es", "fs", "gs",          \
			 "fst0", "fst1", "fst2", "fst3",  \
			 "fst4", "fst5", "fst6", "fst7",  \
			 "fctrl", "fstat", "ftag", "fip", \
			 "fcs", "fdp", "fds",             \
			}

/* Register numbers of various important registers.
   Note that some of these values are "real" register numbers,
   and correspond to the general registers of the machine,
   and some are "phony" register numbers which are too large
   to be actual register numbers as far as the user is concerned
   but do serve to get the desired values when passed to read_register.  */

#define FP_REGNUM 5		/* Contains address of executing stack frame */
#define SP_REGNUM 4		/* Contains address of top of stack */

#define PC_REGNUM 8
#define PS_REGNUM 9

#define FP0_REGNUM 16           /* Floating point register 0 */
#define FPC_REGNUM 24           /* 80387 control register */

/* Total amount of space needed to store our copies of the machine's
   register state, the array `registers'.  */
#define REGISTER_BYTES (16*4 + 8*10 + 7*4)

/* Index within `registers' of the first byte of the space for
   register N.  */

#define REGISTER_BYTE(N) \
    ((N) >= FPC_REGNUM					\
	? (((N) - FPC_REGNUM) * 4) + (16 * 4 + 8 * 10)	\
	: ((N) >= FP0_REGNUM				\
		? (((N) - FP0_REGNUM) * 10) + 16 * 4	\
		: (N) * 4))

/* Number of bytes of storage in the actual machine representation
   for register N.  */

#define REGISTER_RAW_SIZE(N) \
	(((unsigned)((N) - FP0_REGNUM)) < 8 ? 10 : 4)

/* Number of bytes of storage in the program's representation
   for register N. */

#define REGISTER_VIRTUAL_SIZE(N) (((unsigned)((N) - FP0_REGNUM)) < 8 ? 8 : 4)

/* Largest value REGISTER_RAW_SIZE can have.  */

#define MAX_REGISTER_RAW_SIZE 10

/* Largest value REGISTER_VIRTUAL_SIZE can have.  */

#define MAX_REGISTER_VIRTUAL_SIZE sizeof(double)

/* Nonzero if register N requires conversion
   from raw format to virtual format.  */

#define REGISTER_CONVERTIBLE(N) (((unsigned)((N) - FP0_REGNUM)) < 8)

/* Convert data from raw format for register REGNUM in buffer FROM
   to virtual format with type TYPE in buffer TO.  */

#undef REGISTER_CONVERT_TO_VIRTUAL
#define REGISTER_CONVERT_TO_VIRTUAL(REGNUM,TYPE,FROM,TO) \
{ \
  double VAL; \
  i387_to_double ((FROM), (char *) &VAL); \
  store_floating ((TO), TYPE_LENGTH (TYPE), VAL); \
}
extern void
i387_to_double PARAMS ((char *, char *));

/* Convert data from virtual format with type TYPE in buffer FROM
   to raw format for register REGNUM in buffer TO.  */

#undef REGISTER_CONVERT_TO_RAW
#define REGISTER_CONVERT_TO_RAW(TYPE,REGNUM,FROM,TO) \
{ \
  double VAL = extract_floating ((FROM), TYPE_LENGTH (TYPE)); \
  double_to_i387((char *) &VAL, (TO));	\
}
extern void
double_to_i387 PARAMS ((char *, char *));

/* Return the GDB type object for the "standard" data type
   of data in register N.  */

extern struct type *i386_register_virtual_type(int);
#define REGISTER_VIRTUAL_TYPE(N)			\
	(i386_register_virtual_type(N))

/* Store the address of the place in which to copy the structure the
   subroutine will return.  This is called from call_function. */
#if 0
#define STORE_STRUCT_RETURN(ADDR, SP) \
  { (SP) -= sizeof (ADDR);		\
    write_memory ((SP), (char *) &(ADDR), sizeof (ADDR)); }
#else	/* MVS: This is how it should be done for NeXT */
#define STORE_STRUCT_RETURN(ADDR, SP)	write_register(3, ADDR)
#endif

/* Extract from an array REGBUF containing the (raw) register state
   a function return value of type TYPE, and copy that, in virtual format,
   into VALBUF.  Float values are returned as tenbytes in FP0.  */

#ifndef NeXT
#define EXTRACT_RETURN_VALUE(TYPE,REGBUF,VALBUF) \
  memcpy (VALBUF, REGBUF + REGISTER_BYTE (TYPE_CODE (TYPE) == TYPE_CODE_FLT ? FP0_REGNUM : 0), TYPE_LENGTH (TYPE))
#else
/* MVS 3/1/95: the code below will probably not work for cross-debugging.
   In particular it makes certain assumptions about the floating point
   representation.  However if both the host and the inferior use IEEE
   floating point as their native "virtual" format, it should work
   (in particular if both host and inferior are i386 and NeXT)       */

#define EXTRACT_RETURN_VALUE(TYPE,REGBUF,VALBUF)        \
  if (TYPE_CODE(TYPE) == TYPE_CODE_FLT)                 \
  {                                                     \
    REGISTER_CONVERT_TO_VIRTUAL (FP0_REGNUM, (TYPE),    \
      &(REGBUF)[REGISTER_BYTE (FP0_REGNUM)], (VALBUF)); \
  }                                                     \
  else if (TYPE_LENGTH(TYPE) == 8) {      /* 8-byte struct or long long int */\
    *((long *)(VALBUF))     = *((long *)((REGBUF) + REGISTER_BYTE(0))); /*AX*/\
    *((long *)((VALBUF)+4)) = *((long *)((REGBUF) + REGISTER_BYTE(2))); /*DX*/\
  }                                                     \
  else                                                  \
    memcpy ((VALBUF), (REGBUF), TYPE_LENGTH (TYPE))
#endif	/* NeXT */

/* Write into appropriate registers a function return value
   of type TYPE, given in virtual format.  */

#define STORE_RETURN_VALUE(TYPE,VALBUF) \
  write_register_bytes (0, VALBUF, TYPE_LENGTH (TYPE))

/* Extract from an array REGBUF containing the (raw) register state
   the address in which a function should return its structure value,
   as a CORE_ADDR (or an expression that can be used as one).  */

#define EXTRACT_STRUCT_VALUE_ADDRESS(REGBUF) (*(int *)(REGBUF))


/* Describe the pointer in each stack frame to the previous stack frame
   (its caller).  */

/* FRAME_CHAIN takes a frame's nominal address
   and produces the frame's chain-pointer.

   FRAME_CHAIN_COMBINE takes the chain pointer and the frame's nominal address
   and produces the nominal address of the caller frame.

   However, if FRAME_CHAIN_VALID returns zero,
   it means the given frame is the outermost one and has no caller.
   In that case, FRAME_CHAIN_COMBINE is not used.  */

#define FRAME_CHAIN(thisframe) \
   read_memory_integer ((thisframe)->frame, 4)

#define FRAME_CHAIN_VALID(chain, thisframe) (chain)

#define FRAME_CHAIN_COMBINE(chain, thisframe) (chain)

/* Define other aspects of the stack frame.  */

/* A macro that tells us whether the function invocation represented
   by FI does not have a frame on the stack associated with it.  If it
   does not, FRAMELESS is set to 1, else 0.  */
#define FRAMELESS_FUNCTION_INVOCATION(FI, FRAMELESS) \
  (FRAMELESS) = frameless_look_for_prologue(FI)

#define FRAME_SAVED_PC(FRAME) (read_memory_integer ((FRAME)->frame + 4, 4))

#define FRAME_ARGS_ADDRESS(fi) ((fi)->frame)

#define FRAME_LOCALS_ADDRESS(fi) ((fi)->frame)

/* Return number of args passed to a frame.
   Can return -1, meaning no way to tell.  */

#define FRAME_NUM_ARGS(numargs, fi) (numargs) = -1

#ifdef __STDC__		/* Forward decl's for prototypes */
struct frame_info;
struct frame_saved_regs;
#endif

extern int
i386_frame_num_args PARAMS ((struct frame_info *));

/* Return number of bytes at start of arglist that are not really args.  */

#define FRAME_ARGS_SKIP 8

/* Put here the code to store, into a struct frame_saved_regs,
   the addresses of the saved registers of frame described by FRAME_INFO.
   This includes special registers such as pc and fp saved in special
   ways in the stack frame.  sp is even more special:
   the address we return for it IS the sp for the next frame.  */

#define FRAME_FIND_SAVED_REGS(frame_info, frame_saved_regs) \
{ i386_frame_find_saved_regs ((frame_info), &(frame_saved_regs)); }

extern void
i386_frame_find_saved_regs PARAMS ((struct frame_info *,
				    struct frame_saved_regs *));


/* Things needed for making the inferior call functions.  */

/* Push an empty stack frame, to record the current PC, etc.  */

#define PUSH_DUMMY_FRAME { i386_push_dummy_frame (); }

extern void
i386_push_dummy_frame PARAMS ((void));

/* Discard from the stack the innermost frame, restoring all registers.  */

#define POP_FRAME  { i386_pop_frame (); }

extern void
i386_pop_frame PARAMS ((void));

/* this is 
 *   call 11223344 (32 bit relative)
 *   int3
 */

#define CALL_DUMMY { 0x223344e8, 0xcc11 }

#define CALL_DUMMY_LENGTH 8

#define CALL_DUMMY_START_OFFSET 0  /* Start execution at beginning of dummy */

/* Insert the specified number of args and function address
   into a call sequence of the above form stored at DUMMYNAME.  */

#define FIX_CALL_DUMMY(dummyname, pc, fun, nargs, args, type, gcc_p)   \
{ \
	int from, to, delta, loc; \
	loc = (int)(read_register (SP_REGNUM) - CALL_DUMMY_LENGTH); \
	from = loc + 5; \
	to = (int)(fun); \
	delta = to - from; \
	*((char *)(dummyname) + 1) = (delta & 0xff); \
	*((char *)(dummyname) + 2) = ((delta >> 8) & 0xff); \
	*((char *)(dummyname) + 3) = ((delta >> 16) & 0xff); \
	*((char *)(dummyname) + 4) = ((delta >> 24) & 0xff); \
}

#ifndef NeXT
extern void
print_387_control_word PARAMS ((unsigned int));

extern void
print_387_status_word PARAMS ((unsigned int));
#endif	/* NeXT */

/* Offset from SP to first arg on stack at first instruction of a function */

#define SP_ARG0 (1 * 4)

/*
 * Request values for the ptrace system call
 */
enum ptracereq {
        PTRACE_TRACEME = 0,             /* 0, by tracee to begin tracing */
        PTRACE_CHILDDONE = 0,           /* 0, tracee is done with his half */
        PTRACE_PEEKTEXT,                /* 1, read word from text segment */
        PTRACE_PEEKDATA,                /* 2, read word from data segment */
        PTRACE_PEEKUSER,                /* 3, read word from user struct */
        PTRACE_POKETEXT,                /* 4, write word into text segment */
        PTRACE_POKEDATA,                /* 5, write word into data segment */
        PTRACE_POKEUSER,                /* 6, write word into user struct */
        PTRACE_CONT,                    /* 7, continue process */
        PTRACE_KILL,                    /* 8, terminate process */
        PTRACE_SINGLESTEP,              /* 9, single step process */
        PTRACE_ATTACH,                  /* 10, attach to an existing process */
        PTRACE_DETACH,                  /* 11, detach from a process */
        PTRACE_GETREGS,                 /* 12, get all registers */
        PTRACE_SETREGS,                 /* 13, set all registers */
        PTRACE_GETFPREGS,               /* 14, get all floating point regs */
        PTRACE_SETFPREGS,               /* 15, set all floating point regs */
        PTRACE_READDATA,                /* 16, read data segment */
        PTRACE_WRITEDATA,               /* 17, write data segment */
        PTRACE_READTEXT,                /* 18, read text segment */
        PTRACE_WRITETEXT,               /* 19, write text segment */
        PTRACE_GETFPAREGS,              /* 20, get all fpa regs */
        PTRACE_SETFPAREGS,              /* 21, set all fpa regs */
};

#ifdef NeXT	/* used in NeXT-inferior.m */
#define INITIAL_RET_PC() read_memory_integer(read_register(SP_REGNUM), 4)

/* Compiler's internal register numbering scheme */

/* Note that in the stab, gcc has already mapped the register number from
 * its internal scheme to our numbering scheme.
 * We only need this map, therefore, when trying to figure out
 * in what registers the compiler has allocated a value which spans
 * multiple registers.
 */
#define GCC_REMAPPED_REGNUMS(n)	((n)==0 ? 0 : (			\
                                 (n)==1 ? 2 : (			\
                                 (n)==2 ? 1 : (			\
                                 (n)==3 ? 3 : (			\
                                 (n)==4 ? 6 : (			\
                                 (n)==5 ? 7 : (			\
                                 (n)==6 ? 4 : (			\
                                 (n)==7 ? 5 : (			\
				 (n)+4)))))))))
#define NUM_GCC_REMAPPED_REGNUMS (14)


#endif	/* NeXT */

#endif /* !defined (TM_H) */
